import 'dart:async';
import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

const MethodChannel _channel = const MethodChannel('passbase_flutter');

class PassbaseSDK {
  static set prefillUserEmail(String? value) {
    _channel.invokeMethod('setPrefillUserEmail', value);
  }

  static set metaData(String? value) {
    _channel.invokeMethod('setMetaData', value);
  }

  static set prefillCountry(String? value) {
    _channel.invokeMethod('setPrefillCountry', value);
  }

  static Future<void> initialize({required String publishableApiKey}) async {
    await _channel.invokeMethod('initialize', publishableApiKey);
    return;
  }

  static Future<void> startVerification({
    Function? onStart,
    Function(String authenticationKey)? onFinish,
    Function(String errorCode)? onError,
    Function(String identityAccessKey)? onSubmitted,
  }) async {
    _channel.setMethodCallHandler((call) {
      if (call.method == "verification_error") {
        if (onError != null) onError(call.arguments.toString());
      } else if (call.method == "verification_complete") {
        if (onFinish != null) onFinish(call.arguments.toString());
      } else if (call.method == "verification_start") {
        if (onStart != null) onStart();
      } else if (call.method == "verification_submitted") {
        if (onSubmitted != null) onSubmitted(call.arguments.toString());
      }
      return call.arguments;
    });

    await _channel.invokeMethod('startVerification');
  }
}

class PassbaseButton extends StatelessWidget {
  final double? width;
  final double? height;
  final Function(String authenticationKey)? onFinish;
  final Function(String errorCode)? onError;
  final Function? onStart;
  final Function(String identityAccessKey)? onSubmitted;

  PassbaseButton({
    this.width,
    this.height,
    this.onFinish,
    this.onError,
    this.onStart,
    this.onSubmitted,
  }) {
    _channel.setMethodCallHandler((call) {
      if (call.method == "verification_error") {
        if (onError != null) onError!(call.arguments.toString());
      } else if (call.method == "verification_complete") {
        if (onFinish != null) onFinish!(call.arguments.toString());
      } else if (call.method == "verification_start") {
        if (onStart != null) onStart!();
      } else if (call.method == "verification_submitted") {
        if (onSubmitted != null) onSubmitted!(call.arguments.toString());
      }
      return call.arguments;
    });
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      width: width,
      height: height,
      child: Platform.isAndroid
          ? AndroidView(
              viewType: "PassbaseButton",
            )
          : UiKitView(
              viewType: "PassbaseButton",
            ),
    );
  }
}
